@@ -28,7 +28,6 @@ import string
 import os
 import errno
 import sys
-import time
 import getpass
 import logging
 from binascii import hexlify
@@ -36,7 +35,7 @@ from binascii import hexlify
 import duplicity.backend
 from duplicity import globals
 from duplicity import log
-from duplicity.errors import *
+from duplicity.errors import BackendException
 
 read_blocksize=65635            # for doing scp retrievals, where we need to read ourselves
 
@@ -134,7 +133,7 @@ Are you sure you want to continue connecting (yes/no)? """ % (hostname, key.get_
         try:
             if os.path.isfile("/etc/ssh/ssh_known_hosts"):
                 self.client.load_system_host_keys("/etc/ssh/ssh_known_hosts")
-        except Exception, e:
+        except Exception as e:
             raise BackendException("could not load /etc/ssh/ssh_known_hosts, maybe corrupt?")
         try:
             # use load_host_keys() to signal it's writable to paramiko
@@ -144,7 +143,7 @@ Are you sure you want to continue connecting (yes/no)? """ % (hostname, key.get_
                 self.client.load_host_keys(file)
             else:
                 self.client._host_keys_filename = file
-        except Exception, e:
+        except Exception as e:
             raise BackendException("could not load ~/.ssh/known_hosts, maybe corrupt?")
 
         """ the next block reorganizes all host parameters into a
@@ -211,7 +210,7 @@ Are you sure you want to continue connecting (yes/no)? """ % (hostname, key.get_
                                 allow_agent=True, 
                                 look_for_keys=True,
                                 key_filename=self.config['identityfile'])
-        except Exception, e:
+        except Exception as e:
             raise BackendException("ssh connection to %s@%s:%d failed: %s" % (
                                     self.config['user'],
                                     self.config['hostname'],
@@ -229,10 +228,9 @@ Are you sure you want to continue connecting (yes/no)? """ % (hostname, key.get_
         else:
             try:
                 self.sftp=self.client.open_sftp()
-            except Exception, e:
+            except Exception as e:
                 raise BackendException("sftp negotiation failed: %s" % e)
 
-
             # move to the appropriate directory, possibly after creating it and its parents
             dirs = self.remote_dir.split(os.sep)
             if len(dirs) > 0:
@@ -244,170 +242,104 @@ Are you sure you want to continue connecting (yes/no)? """ % (hostname, key.get_
                         continue
                     try:
                         attrs=self.sftp.stat(d)
-                    except IOError, e:
+                    except IOError as e:
                         if e.errno == errno.ENOENT:
                             try:
                                 self.sftp.mkdir(d)
-                            except Exception, e:
+                            except Exception as e:
                                 raise BackendException("sftp mkdir %s failed: %s" % (self.sftp.normalize(".")+"/"+d,e))
                         else:
                             raise BackendException("sftp stat %s failed: %s" % (self.sftp.normalize(".")+"/"+d,e))
                     try:
                         self.sftp.chdir(d)
-                    except Exception, e:
+                    except Exception as e:
                         raise BackendException("sftp chdir to %s failed: %s" % (self.sftp.normalize(".")+"/"+d,e))
 
-    def put(self, source_path, remote_filename = None):
-        """transfers a single file to the remote side.
-        In scp mode unavoidable quoting issues will make this fail if the remote directory or file name
-        contain single quotes."""
-        if not remote_filename:
-            remote_filename = source_path.get_filename()
-        
-        for n in range(1, globals.num_retries+1):
-            if n > 1:
-                # sleep before retry
-                time.sleep(self.retry_delay)
+    def _put(self, source_path, remote_filename):
+        if globals.use_scp:
+            f=file(source_path.name,'rb')
             try:
-                if (globals.use_scp):
-                    f=file(source_path.name,'rb')
-                    try:
-                        chan=self.client.get_transport().open_session()
-                        chan.settimeout(globals.timeout)
-                        chan.exec_command("scp -t '%s'" % self.remote_dir) # scp in sink mode uses the arg as base directory
-                    except Exception, e:
-                        raise BackendException("scp execution failed: %s" % e)
-                    # scp protocol: one 0x0 after startup, one after the Create meta, one after saving
-                    # if there's a problem: 0x1 or 0x02 and some error text
-                    response=chan.recv(1)
-                    if (response!="\0"):
-                        raise BackendException("scp remote error: %s" % chan.recv(-1))
-                    fstat=os.stat(source_path.name)
-                    chan.send('C%s %d %s\n' %(oct(fstat.st_mode)[-4:], fstat.st_size, remote_filename))
-                    response=chan.recv(1)
-                    if (response!="\0"):
-                        raise BackendException("scp remote error: %s" % chan.recv(-1))
-                    chan.sendall(f.read()+'\0')
-                    f.close()
-                    response=chan.recv(1)
-                    if (response!="\0"):
-                        raise BackendException("scp remote error: %s" % chan.recv(-1))
-                    chan.close()
-                    return
-                else:
-                    try:
-                        self.sftp.put(source_path.name,remote_filename)
-                        return
-                    except Exception, e:
-                        raise BackendException("sftp put of %s (as %s) failed: %s" % (source_path.name,remote_filename,e))
-            except Exception, e:
-                log.Warn("%s (Try %d of %d) Will retry in %d seconds." % (e,n,globals.num_retries,self.retry_delay))
-        raise BackendException("Giving up trying to upload '%s' after %d attempts" % (remote_filename,n))
+                chan=self.client.get_transport().open_session()
+                chan.settimeout(globals.timeout)
+                chan.exec_command("scp -t '%s'" % self.remote_dir) # scp in sink mode uses the arg as base directory
+            except Exception as e:
+                raise BackendException("scp execution failed: %s" % e)
+            # scp protocol: one 0x0 after startup, one after the Create meta, one after saving
+            # if there's a problem: 0x1 or 0x02 and some error text
+            response=chan.recv(1)
+            if (response!="\0"):
+                raise BackendException("scp remote error: %s" % chan.recv(-1))
+            fstat=os.stat(source_path.name)
+            chan.send('C%s %d %s\n' %(oct(fstat.st_mode)[-4:], fstat.st_size, remote_filename))
+            response=chan.recv(1)
+            if (response!="\0"):
+                raise BackendException("scp remote error: %s" % chan.recv(-1))
+            chan.sendall(f.read()+'\0')
+            f.close()
+            response=chan.recv(1)
+            if (response!="\0"):
+                raise BackendException("scp remote error: %s" % chan.recv(-1))
+            chan.close()
+        else:
+            self.sftp.put(source_path.name,remote_filename)
 
-
-    def get(self, remote_filename, local_path):
-        """retrieves a single file from the remote side.
-        In scp mode unavoidable quoting issues will make this fail if the remote directory or file names
-        contain single quotes."""
-        
-        for n in range(1, globals.num_retries+1):
-            if n > 1:
-                # sleep before retry
-                time.sleep(self.retry_delay)
+    def _get(self, remote_filename, local_path):
+        if globals.use_scp:
             try:
-                if (globals.use_scp):
-                    try:
-                        chan=self.client.get_transport().open_session()
-                        chan.settimeout(globals.timeout)
-                        chan.exec_command("scp -f '%s/%s'" % (self.remote_dir,remote_filename))
-                    except Exception, e:
-                        raise BackendException("scp execution failed: %s" % e)
+                chan=self.client.get_transport().open_session()
+                chan.settimeout(globals.timeout)
+                chan.exec_command("scp -f '%s/%s'" % (self.remote_dir,remote_filename))
+            except Exception as e:
+                raise BackendException("scp execution failed: %s" % e)
 
-                    chan.send('\0')     # overall ready indicator
-                    msg=chan.recv(-1)
-                    m=re.match(r"C([0-7]{4})\s+(\d+)\s+(\S.*)$",msg)
-                    if (m==None or m.group(3)!=remote_filename):
-                        raise BackendException("scp get %s failed: incorrect response '%s'" % (remote_filename,msg))
-                    chan.recv(1)        # dispose of the newline trailing the C message
+            chan.send('\0')     # overall ready indicator
+            msg=chan.recv(-1)
+            m=re.match(r"C([0-7]{4})\s+(\d+)\s+(\S.*)$",msg)
+            if (m==None or m.group(3)!=remote_filename):
+                raise BackendException("scp get %s failed: incorrect response '%s'" % (remote_filename,msg))
+            chan.recv(1)        # dispose of the newline trailing the C message
 
-                    size=int(m.group(2))
-                    togo=size
-                    f=file(local_path.name,'wb')
-                    chan.send('\0')     # ready for data
-                    try:
-                        while togo>0:
-                            if togo>read_blocksize:
-                                blocksize = read_blocksize
-                            else:
-                                blocksize = togo
-                            buff=chan.recv(blocksize)
-                            f.write(buff)
-                            togo-=len(buff)
-                    except Exception, e:
-                        raise BackendException("scp get %s failed: %s" % (remote_filename,e))
+            size=int(m.group(2))
+            togo=size
+            f=file(local_path.name,'wb')
+            chan.send('\0')     # ready for data
+            try:
+                while togo>0:
+                    if togo>read_blocksize:
+                        blocksize = read_blocksize
+                    else:
+                        blocksize = togo
+                    buff=chan.recv(blocksize)
+                    f.write(buff)
+                    togo-=len(buff)
+            except Exception as e:
+                raise BackendException("scp get %s failed: %s" % (remote_filename,e))
 
-                    msg=chan.recv(1)    # check the final status
-                    if msg!='\0':
-                        raise BackendException("scp get %s failed: %s" % (remote_filename,chan.recv(-1)))
-                    f.close()
-                    chan.send('\0')     # send final done indicator
-                    chan.close()
-                    return
-                else:
-                    try:
-                        self.sftp.get(remote_filename,local_path.name)
-                        return
-                    except Exception, e:
-                        raise BackendException("sftp get of %s (to %s) failed: %s" % (remote_filename,local_path.name,e))
-                local_path.setdata()
-            except Exception, e:
-                log.Warn("%s (Try %d of %d) Will retry in %d seconds." % (e,n,globals.num_retries,self.retry_delay))
-        raise BackendException("Giving up trying to download '%s' after %d attempts" % (remote_filename,n))
+            msg=chan.recv(1)    # check the final status
+            if msg!='\0':
+                raise BackendException("scp get %s failed: %s" % (remote_filename,chan.recv(-1)))
+            f.close()
+            chan.send('\0')     # send final done indicator
+            chan.close()
+        else:
+            self.sftp.get(remote_filename,local_path.name)
 
     def _list(self):
-        """lists the contents of the one-and-only duplicity dir on the remote side.
-        In scp mode unavoidable quoting issues will make this fail if the directory name
-        contains single quotes."""
-        for n in range(1, globals.num_retries+1):
-            if n > 1:
-                # sleep before retry
-                time.sleep(self.retry_delay)
-            try:
-                if (globals.use_scp):
-                    output=self.runremote("ls -1 '%s'" % self.remote_dir,False,"scp dir listing ")
-                    return output.splitlines()
-                else:
-                    try:
-                        return self.sftp.listdir()
-                    except Exception, e:
-                        raise BackendException("sftp listing of %s failed: %s" % (self.sftp.getcwd(),e))
-            except Exception, e:
-                log.Warn("%s (Try %d of %d) Will retry in %d seconds." % (e,n,globals.num_retries,self.retry_delay))
-        raise BackendException("Giving up trying to list '%s' after %d attempts" % (self.remote_dir,n))
+        # In scp mode unavoidable quoting issues will make this fail if the
+        # directory name contains single quotes.
+        if globals.use_scp:
+            output = self.runremote("ls -1 '%s'" % self.remote_dir, False, "scp dir listing ")
+            return output.splitlines()
+        else:
+            return self.sftp.listdir()
 
-    def delete(self, filename_list):
-        """deletes all files in the list on the remote side. In scp mode unavoidable quoting issues
-        will cause failures if filenames containing single quotes are encountered."""
-        for fn in filename_list:
-            # Try to delete each file several times before giving up completely.
-            for n in range(1, globals.num_retries+1):
-                try:
-                    if (globals.use_scp):
-                        self.runremote("rm '%s/%s'" % (self.remote_dir,fn),False,"scp rm ")
-                    else:
-                        try:
-                            self.sftp.remove(fn)
-                        except Exception, e:
-                            raise BackendException("sftp rm %s failed: %s" % (fn,e))
-
-                    # If we get here, we deleted this file successfully. Move on to the next one.
-                    break
-                except Exception, e:
-                    if n == globals.num_retries:
-                        log.FatalError(str(e), log.ErrorCode.backend_error)
-                    else:
-                        log.Warn("%s (Try %d of %d) Will retry in %d seconds." % (e,n,globals.num_retries,self.retry_delay))
-                        time.sleep(self.retry_delay)
+    def _delete(self, filename):
+        # In scp mode unavoidable quoting issues will cause failures if
+        # filenames containing single quotes are encountered.
+        if globals.use_scp:
+            self.runremote("rm '%s/%s'" % (self.remote_dir, filename), False, "scp rm ")
+        else:
+            self.sftp.remove(filename)
 
     def runremote(self,cmd,ignoreexitcode=False,errorprefix=""):
         """small convenience function that opens a shell channel, runs remote command and returns
@@ -416,7 +348,7 @@ Are you sure you want to continue connecting (yes/no)? """ % (hostname, key.get_
             chan=self.client.get_transport().open_session()
             chan.settimeout(globals.timeout)
             chan.exec_command(cmd)
-        except Exception, e:
+        except Exception as e:
             raise BackendException("%sexecution failed: %s" % (errorprefix,e))
         output=chan.recv(-1)
         res=chan.recv_exit_status()
@@ -434,11 +366,7 @@ Are you sure you want to continue connecting (yes/no)? """ % (hostname, key.get_
         sshconfig = paramiko.SSHConfig()
         try:
             sshconfig.parse(open(file))
-        except Exception, e:
+        except Exception as e:
             raise BackendException("could not load '%s', maybe corrupt?" % (file))
         
         return sshconfig.lookup(host)
-
-duplicity.backend.register_backend("sftp", SSHParamikoBackend)
-duplicity.backend.register_backend("scp", SSHParamikoBackend)
-duplicity.backend.register_backend("ssh", SSHParamikoBackend)
