@@ -22,18 +22,12 @@
 
 import duplicity.backend
 from duplicity import globals
-import sys
 
 if globals.s3_use_multiprocessing:
-    if sys.version_info[:2] < (2, 6):
-        print "Sorry, S3 multiprocessing requires version 2.6 or later of python"
-        sys.exit(1)
-    from _boto_multi import BotoBackend as BotoMultiUploadBackend
-    duplicity.backend.register_backend("gs", BotoMultiUploadBackend)
-    duplicity.backend.register_backend("s3", BotoMultiUploadBackend)
-    duplicity.backend.register_backend("s3+http", BotoMultiUploadBackend)
+    from ._boto_multi import BotoBackend
 else:
-    from _boto_single import BotoBackend as BotoSingleUploadBackend
-    duplicity.backend.register_backend("gs", BotoSingleUploadBackend)
-    duplicity.backend.register_backend("s3", BotoSingleUploadBackend)
-    duplicity.backend.register_backend("s3+http", BotoSingleUploadBackend)
+    from ._boto_single import BotoBackend
+
+duplicity.backend.register_backend("gs", BotoBackend)
+duplicity.backend.register_backend("s3", BotoBackend)
+duplicity.backend.register_backend("s3+http", BotoBackend)
