@@ -21,6 +21,8 @@
 
 """Parse command line, check for consistency, and set globals"""
 
+from future_builtins import filter
+
 from copy import copy
 import optparse
 import os
@@ -110,7 +112,7 @@ def check_file(option, opt, value):
 def check_time(option, opt, value):
     try:
         return dup_time.genstrtotime(value)
-    except dup_time.TimeException, e:
+    except dup_time.TimeException as e:
         raise optparse.OptionValueError(str(e))
 
 def check_verbosity(option, opt, value):
@@ -208,13 +210,6 @@ def parse_cmdline_options(arglist):
     global select_opts, select_files, full_backup
     global list_current, collection_status, cleanup, remove_time, verify
 
-    def use_gio(*args):
-        try:
-            import duplicity.backends.giobackend
-            backend.force_backend(duplicity.backends.giobackend.GIOBackend)
-        except ImportError:
-            log.FatalError(_("Unable to load gio backend: %s") % str(sys.exc_info()[1]), log.ErrorCode.gio_not_available)
-
     def set_log_fd(fd):
         if fd < 1:
             raise optparse.OptionValueError("log-fd must be greater than zero.")
@@ -330,7 +325,7 @@ def parse_cmdline_options(arglist):
 
     # used to provide a prefix on top of the defaul tar file name
     parser.add_option("--file-prefix", type = "string", dest = "file_prefix", action = "store")
-    
+
     # used to provide a suffix for manifest files only
     parser.add_option("--file-prefix-manifest", type = "string", dest = "file_prefix_manifest", action = "store")
 
@@ -363,7 +358,9 @@ def parse_cmdline_options(arglist):
     # the time specified
     parser.add_option("--full-if-older-than", type = "time", dest = "full_force_time", metavar = _("time"))
 
-    parser.add_option("--gio", action = "callback", callback = use_gio)
+    parser.add_option("--gio",action = "callback", dest = "use_gio",
+                      callback = lambda o, s, v, p: (setattr(p.values, o.dest, True),
+                                                     old_fn_deprecation(s)))
 
     parser.add_option("--gpg-options", action = "extend", metavar = _("options"))
 
@@ -382,7 +379,7 @@ def parse_cmdline_options(arglist):
                       callback = lambda o, s, v, p: (log.Warn(
                           _("Running in 'ignore errors' mode due to %s; please "
                             "re-consider if this was not intended") % s),
-                          setattr(p.values, "ignore errors", True)))
+                          setattr(p.values, "ignore_errors", True)))
 
     # Whether to use the full email address as the user name when
     # logging into an imap server. If false just the user name
@@ -452,7 +449,10 @@ def parse_cmdline_options(arglist):
                                                    old_fn_deprecation(s)))
 
     # Level of Redundancy in % for Par2 files
-    parser.add_option("--par2-redundancy", type="int", metavar=_("number"))
+    parser.add_option("--par2-redundancy", type = "int", metavar = _("number"))
+
+    # Verbatim par2 options
+    parser.add_option("--par2-options", action = "extend", metavar = _("options"))
 
     # Used to display the progress for the full and incremental backup operations
     parser.add_option("--progress", action = "store_true")
@@ -508,9 +508,7 @@ def parse_cmdline_options(arglist):
     parser.add_option("--s3_multipart_max_timeout", type="int", metavar=_("number"))
 
     # Option to allow the s3/boto backend use the multiprocessing version.
-    # By default it is off since it does not work for Python 2.4 or 2.5.
-    if sys.version_info[:2] >= (2, 6):
-        parser.add_option("--s3-use-multiprocessing", action = "store_true")
+    parser.add_option("--s3-use-multiprocessing", action = "store_true")
 
     # Option to allow use of server side encryption in s3
     parser.add_option("--s3-use-server-side-encryption", action="store_true", dest="s3_use_sse")
@@ -521,7 +519,7 @@ def parse_cmdline_options(arglist):
     # sftp command to use (ssh pexpect backend)
     parser.add_option("--sftp-command", metavar = _("command"))
 
-    # sftp command to use (ssh pexpect backend)
+    # allow the user to switch cloudfiles backend
     parser.add_option("--cf-backend", metavar = _("pyrax|cloudfiles"))
 
     # If set, use short (< 30 char) filenames for all the remote files.
@@ -864,6 +862,7 @@ def usage():
   webdavs://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
   gdocs://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
   mega://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
+  copy://%(user)s[:%(password)s]@%(other_host)s/%(some_dir)s
   dpbx:///%(some_dir)s
 
 """ % dict
