@@ -27,6 +27,8 @@ first, the signature or delta is constructed of a ROPath iterator.  In
 the second, the ROPath iterator is put into tar block form.
 """
 
+from future_builtins import map
+
 import cStringIO, types, math
 from duplicity import statistics
 from duplicity import util
@@ -79,8 +81,8 @@ def DirDelta(path_iter, dirsig_fileobj_list):
     global stats
     stats = statistics.StatsDeltaProcess()
     if type(dirsig_fileobj_list) is types.ListType:
-        sig_iter = combine_path_iters(map(sigtar2path_iter,
-                                          dirsig_fileobj_list))
+        sig_iter = combine_path_iters([sigtar2path_iter(x) for x
+                                       in dirsig_fileobj_list])
     else:
         sig_iter = sigtar2path_iter(dirsig_fileobj_list)
     delta_iter = get_delta_iter(path_iter, sig_iter)
@@ -100,7 +102,7 @@ def delta_iter_error_handler(exc, new_path, sig_path, sig_tar = None):
         index_string = sig_path.get_relative_path()
     else:
         assert 0, "Both new and sig are None for some reason"
-    log.Warn(_("Error %s getting delta for %s") % (str(exc), index_string))
+    log.Warn(_("Error %s getting delta for %s") % (str(exc), util.ufn(index_string)))
     return None
 
 
@@ -160,14 +162,14 @@ def log_delta_path(delta_path, new_path = None, stats = None):
         if new_path and stats:
             stats.add_new_file(new_path)
         log.Info(_("A %s") %
-                 (delta_path.get_relative_path(),),
+                 (util.ufn(delta_path.get_relative_path())),
                  log.InfoCode.diff_file_new,
                  util.escape(delta_path.get_relative_path()))
     else:
         if new_path and stats:
             stats.add_changed_file(new_path)
         log.Info(_("M %s") %
-                 (delta_path.get_relative_path(),),
+                 (util.ufn(delta_path.get_relative_path())),
                  log.InfoCode.diff_file_changed,
                  util.escape(delta_path.get_relative_path()))
 
@@ -188,8 +190,8 @@ def get_delta_iter(new_iter, sig_iter, sig_fileobj=None):
     else:
         sigTarFile = None
     for new_path, sig_path in collated:
-        log.Debug(_("Comparing %s and %s") % (new_path and new_path.index,
-                                              sig_path and sig_path.index))
+        log.Debug(_("Comparing %s and %s") % (new_path and util.uindex(new_path.index),
+                                              sig_path and util.uindex(sig_path.index)))
         if not new_path or not new_path.type:
             # File doesn't exist (but ignore attempts to delete base dir;
             # old versions of duplicity could have written out the sigtar in
@@ -197,7 +199,7 @@ def get_delta_iter(new_iter, sig_iter, sig_fileobj=None):
             if sig_path and sig_path.exists() and sig_path.index != ():
                 # but signature says it did
                 log.Info(_("D %s") %
-                         (sig_path.get_relative_path(),),
+                         (util.ufn(sig_path.get_relative_path())),
                          log.InfoCode.diff_file_deleted,
                          util.escape(sig_path.get_relative_path()))
                 if sigTarFile:
@@ -342,8 +344,7 @@ def combine_path_iters(path_iter_list):
             else:
                 break # assumed triple_list sorted, so can exit now
 
-    triple_list = filter(lambda x: x, map(get_triple,
-                                          range(len(path_iter_list))))
+    triple_list = [x for x in map(get_triple, range(len(path_iter_list))) if x]
     while triple_list:
         triple_list.sort()
         yield triple_list[0][2]
@@ -375,7 +376,7 @@ def get_combined_path_iter(sig_infp_list):
     """
     Return path iter combining signatures in list of open sig files
     """
-    return combine_path_iters(map(sigtar2path_iter, sig_infp_list))
+    return combine_path_iters([sigtar2path_iter(x) for x in sig_infp_list])
 
 
 class FileWithReadCounter:
@@ -389,9 +390,9 @@ class FileWithReadCounter:
     def read(self, length = -1):
         try:
             buf = self.infile.read(length)
-        except IOError, ex:
+        except IOError as ex:
             buf = ""
-            log.Warn(_("Error %s getting delta for %s") % (str(ex), self.infile.name))
+            log.Warn(_("Error %s getting delta for %s") % (str(ex), util.ufn(self.infile.name)))
         if stats:
             stats.SourceFileSize += len(buf)
         return buf
@@ -461,7 +462,7 @@ class TarBlockIter:
         TarBlockIter initializer
         """
         self.input_iter = input_iter
-        self.offset = 0l                    # total length of data read
+        self.offset = 0                     # total length of data read
         self.process_waiting = False        # process_continued has more blocks
         self.process_next_vol_number = None # next volume number to write in multivol
         self.previous_index = None          # holds index of last block returned
@@ -469,13 +470,14 @@ class TarBlockIter:
         self.remember_next = False          # see remember_next_index()
         self.remember_value = None          # holds index of next block
         self.remember_block = None          # holds block of next block
+        self.queued_data = None             # data to return in next next() call
 
     def tarinfo2tarblock(self, index, tarinfo, file_data = ""):
         """
         Make tarblock out of tarinfo and file data
         """
         tarinfo.size = len(file_data)
-        headers = tarinfo.tobuf()
+        headers = tarinfo.tobuf(errors='replace')
         blocks, remainder = divmod(tarinfo.size, tarfile.BLOCKSIZE) #@UnusedVariable
         if remainder > 0:
             filler_data = "\0" * (tarfile.BLOCKSIZE - remainder)
@@ -504,6 +506,12 @@ class TarBlockIter:
         """
         Return next block and update offset
         """
+        if self.queued_data is not None:
+            result = self.queued_data
+            self.queued_data = None
+            # Keep rest of metadata as is (like previous_index)
+            return result
+
         if self.process_waiting:
             result = self.process_continued()
         else:
@@ -532,6 +540,12 @@ class TarBlockIter:
         """
         return self.previous_index, self.previous_block
 
+    def queue_index_data(self, data):
+        """
+        Next time next() is called, we will return data instead of processing
+        """
+        self.queued_data = data
+
     def remember_next_index(self):
         """
         When called, remember the index of the next block iterated
@@ -551,7 +565,7 @@ class TarBlockIter:
         Return closing string for tarfile, reset offset
         """
         blocks, remainder = divmod(self.offset, tarfile.RECORDSIZE) #@UnusedVariable
-        self.offset = 0l
+        self.offset = 0
         return '\0' * (tarfile.RECORDSIZE - remainder) # remainder can be 0
 
     def __iter__(self):
@@ -723,5 +737,5 @@ def get_block_size(file_len):
         return 512 # set minimum of 512 bytes
     else:
         # Split file into about 2000 pieces, rounding to 512
-        file_blocksize = long((file_len / (2000 * 512)) * 512)
+        file_blocksize = int((file_len / (2000 * 512)) * 512)
         return min(file_blocksize, globals.max_blocksize)
