@@ -26,7 +26,7 @@ import socket
 
 
 # The current version of duplicity
-version = "0.6.26"
+version = "0.7.0"
 
 # Prefix for all files (appended before type-specific prefixes)
 file_prefix = ""
@@ -87,7 +87,7 @@ gpg_profile = None
 gpg_options = ''
 
 # Maximum file blocksize
-max_blocksize = 2048L
+max_blocksize = 2048
 
 # If true, filelists and directory statistics will be split on
 # nulls instead of newlines.
@@ -287,3 +287,6 @@ par2_redundancy = 10
 
 # Verbatim par2 other options
 par2_options = ""
+
+# Whether to enable gio backend
+use_gio = False
