@@ -19,6 +19,8 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
+from future_builtins import filter, map
+
 import re #@UnusedImport
 import types
 import os
@@ -503,7 +505,7 @@ def integrate_patch_iters( iter_list ):
             if final_ropath.exists():
                 # otherwise final patch was delete
                 yield final_ropath
-        except Exception, e:
+        except Exception as e:
             filename = normalized[-1].get_ropath().get_relative_path()
             log.Warn(_("Error '%s' patching %s") % 
                      (util.uexc(e), util.ufn(filename)),
@@ -517,11 +519,10 @@ def tarfiles2rop_iter( tarfile_list, restrict_index=() ):
     the restrict_index.
 
     """
-    diff_iters = map( difftar2path_iter, tarfile_list )
+    diff_iters = [difftar2path_iter(x) for x in tarfile_list]
     if restrict_index:
         # Apply filter before integration
-        diff_iters = map( lambda i: filter_path_iter( i, restrict_index ),
-                         diff_iters )
+        diff_iters = [filter_path_iter(x, restrict_index) for x in diff_iters]
     return integrate_patch_iters( diff_iters )
 
 def Write_ROPaths( base_path, rop_iter ):
