@@ -26,6 +26,8 @@ associates stat information with filenames
 
 """
 
+from future_builtins import filter
+
 import stat, errno, socket, time, re, gzip
 
 from duplicity import tarfile
@@ -93,8 +95,13 @@ class ROPath:
 
         self.mode = stat.S_IMODE(st_mode)
         if self.type in ("chr", "blk"):
-            self.devnums = (os.major(self.stat.st_rdev),
-                            os.minor(self.stat.st_rdev))
+            try:
+                self.devnums = (os.major(self.stat.st_rdev),
+                                os.minor(self.stat.st_rdev))
+            except:
+                log.Warn(_("Warning: %s invalid devnums (0x%X), treating as (0, 0).")
+                         % (util.ufn(self.get_relative_path()), self.stat.st_rdev))
+                self.devnums = (0, 0)
 
     def blank(self):
         """Black out self - set type and stat to None"""
@@ -500,7 +507,7 @@ class Path(ROPath):
         """Refresh stat cache"""
         try:
             self.stat = os.lstat(self.name)
-        except OSError, e:
+        except OSError as e:
             err_string = errno.errorcode[e[0]]
             if err_string in ["ENOENT", "ENOTDIR", "ELOOP", "ENOTCONN"]:
                 self.stat, self.type = None, None # file doesn't exist
