@@ -32,6 +32,9 @@ and the time left to transfer all the (yet unknown) amount of data to send.
 This is a forecast based on gathered evidence.
 """
 
+from __future__ import absolute_import
+
+import collections as sys_collections
 import math
 import threading
 import time
@@ -41,29 +44,6 @@ from duplicity import log
 import pickle
 import os
 
-def import_non_local(name, custom_name=None):
-    """
-    This function is needed to play a trick... as there exists a local
-    "collections" module, that is named the same as a system module
-    """
-    import imp, sys
-
-    custom_name = custom_name or name
-
-    f, pathname, desc = imp.find_module(name, sys.path[1:])
-    module = imp.load_module(custom_name, f, pathname, desc)
-    f.close()
-
-    return module
-
-"""
-Import non-local module, use a custom name to differentiate it from local
-This name is only used internally for identifying the module. We decide
-the name in the local scope by assigning it to the variable sys_collections.
-"""
-sys_collections = import_non_local('collections','sys_collections')
-
-
 tracker = None
 progress_thread = None
 
@@ -123,7 +103,7 @@ class Snapshot(sys_collections.deque):
 
 
 
-class ProgressTracker:
+class ProgressTracker():
 
     def __init__(self):
         self.total_stats = None
@@ -262,7 +242,7 @@ class ProgressTracker:
         projection = 1.0
         if self.progress_estimation > 0:
             projection = (1.0 - self.progress_estimation) / self.progress_estimation
-        self.time_estimation = long(projection * float(self.elapsed_sum.total_seconds()))
+        self.time_estimation = int(projection * float(self.elapsed_sum.total_seconds()))
 
         # Apply values only when monotonic, so the estimates look more consistent to the human eye
         if self.progress_estimation < last_progress_estimation:
@@ -297,7 +277,7 @@ class ProgressTracker:
         volume and for the current volume
         """
         changing = max(bytecount - self.last_bytecount, 0)
-        self.total_bytecount += long(changing) # Annotate only changing bytes since last probe
+        self.total_bytecount += int(changing) # Annotate only changing bytes since last probe
         self.last_bytecount = bytecount
         if changing > 0:
             self.stall_last_time = datetime.now()
