@@ -19,6 +19,8 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
+from future_builtins import filter, map
+
 import os #@UnusedImport
 import re #@UnusedImport
 import stat #@UnusedImport
@@ -235,8 +237,8 @@ class Select:
                         filelists[filelists_index], 0, arg))
                     filelists_index += 1
                 elif opt == "--exclude-globbing-filelist":
-                    map(self.add_selection_func,
-                        self.filelist_globbing_get_sfs(filelists[filelists_index], 0, arg))
+                    for sf in self.filelist_globbing_get_sfs(filelists[filelists_index], 0, arg):
+                        self.add_selection_func(sf)
                     filelists_index += 1
                 elif opt == "--exclude-other-filesystems":
                     self.add_selection_func(self.other_filesystems_get_sf(0))
@@ -249,14 +251,14 @@ class Select:
                         filelists[filelists_index], 1, arg))
                     filelists_index += 1
                 elif opt == "--include-globbing-filelist":
-                    map(self.add_selection_func,
-                        self.filelist_globbing_get_sfs(filelists[filelists_index], 1, arg))
+                    for sf in self.filelist_globbing_get_sfs(filelists[filelists_index], 1, arg):
+                        self.add_selection_func(sf)
                     filelists_index += 1
                 elif opt == "--include-regexp":
                     self.add_selection_func(self.regexp_get_sf(arg, 1))
                 else:
                     assert 0, "Bad selection option %s" % opt
-        except SelectError, e:
+        except SelectError as e:
             self.parse_catch_error(e)
         assert filelists_index == len(filelists)
         self.parse_last_excludes()
@@ -351,7 +353,7 @@ probably isn't what you meant.""") %
                 continue # skip blanks
             try:
                 tuple = self.filelist_parse_line(line, include)
-            except FilePrefixError, exc:
+            except FilePrefixError as exc:
                 incr_warnings(exc)
                 continue
             tuple_list.append(tuple)
@@ -626,8 +628,7 @@ probably isn't what you meant.""") %
             raise GlobbingError("Consecutive '/'s found in globbing string "
                                 + glob_str)
 
-        prefixes = map(lambda i: "/".join(glob_parts[:i+1]),
-                       range(len(glob_parts)))
+        prefixes = ["/".join(glob_parts[:i+1]) for i in range(len(glob_parts))]
         # we must make exception for root "/", only dir to end in slash
         if prefixes[0] == "":
             prefixes[0] = "/"
