@@ -23,6 +23,8 @@
 Miscellaneous utilities.
 """
 
+from future_builtins import map
+
 import errno
 import os
 import sys
@@ -86,7 +88,7 @@ def maybe_ignore_errors(fn):
     """
     try:
         return fn()
-    except Exception, e:
+    except Exception as e:
         if globals.ignore_errors:
             log.Warn(_("IGNORED_ERROR: Warning: ignoring error as requested: %s: %s")
                      % (e.__class__.__name__, uexc(e)))
@@ -137,7 +139,7 @@ def ignore_missing(fn, filename):
     """
     try:
         fn(filename)
-    except OSError, ex:
+    except OSError as ex:
         if ex.errno == errno.ENOENT:
             pass
         else:
